/**
 * Copyright (c) 2008-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.registry.core.repository.jdbc;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.ow2.petals.registry.api.Query;
import org.ow2.petals.registry.api.Resource;
import org.ow2.petals.registry.api.context.Context;
import org.ow2.petals.registry.api.exception.LifeCycleException;
import org.ow2.petals.registry.api.exception.RegistryException;
import org.ow2.petals.registry.core.repository.AbstractRepository;
import org.ow2.petals.registry.core.repository.bo.Endpoint;
import org.ow2.petals.registry.core.repository.bo.TOAdapter;
import org.ow2.petals.registry.core.repository.dao.DAOException;
import org.ow2.petals.registry.core.repository.dao.EndpointDAO;
import org.ow2.petals.registry.core.repository.hibernate.HibernateHelper;

/**
 * The JDBC repository implementation.
 * <p>
 * All the transaction stuff is done in the
 * {@link AbstractRepository}. The only thing that has to be done here is to
 * call the good DAOs... If the repository is configured to work with an
 * embedded JDBC server it is instanciated and its lifecycle is managed inside
 * the repository lifecycle.
 * </p>
 * 
 * @author Christophe HAMERLING - EBM WebSourcing
 */
public class JDBCRepository extends AbstractRepository {

    protected EndpointDAO endpointDAO;

    protected TOAdapter adapter;
    
    protected HibernateHelper hibernateHelper;

    protected Log log = LogFactory.getLog(JDBCRepository.class);

    public JDBCRepository() {
        super();
    }

    @Override
    public void createLifeCycle() {
        this.lifeCycle = new JDBCRepositoryLifeCycle(this);
    }

    @Override
    public void init(final Context context) throws LifeCycleException {
        super.init(context);
        this.endpointDAO = new EndpointDAO();
        this.adapter = new TOAdapter();
    }

    @Override
    protected boolean doDelete(final String path) throws RegistryException {
        boolean result = false;
        Session session = hibernateHelper.getSession();
        try {
            result = this.endpointDAO.delete(session, path);
        } catch (DAOException e) {
            throw new RegistryException(e.getMessage());
        } finally {
            if (session != null) {
                try {
                    session.disconnect();
                } catch (HibernateException e) {
                }
            }
        }
        return result;
    }

    @Override
    protected Resource doGet(final String path) throws RegistryException {
        Resource result = null;
        Endpoint edp;
        Session session = hibernateHelper.getSession();

        try {
            edp = this.endpointDAO.get(session, path);
        } catch (DAOException e) {
            throw new RegistryException(e.getMessage());
        } finally {
            if (session != null) {
                try {
                    session.disconnect();
                } catch (HibernateException e) {
                }
            }
        }
        if (edp != null) {
            result = this.adapter.toTO(edp);
        }
        return result;
    }

    @Override
    protected boolean doPut(final String path, final Resource resource) throws RegistryException {
        boolean result = true;
        Session session = hibernateHelper.getSession();
        try {

            Endpoint edp = this.adapter.toBO(resource);
            if (this.endpointDAO.endpointExists(session, path)) {
                if (this.log.isDebugEnabled()) {
                    this.log.debug("Endpoint already exists under " + path);
                }
                this.endpointDAO.update(session, edp, path);
            } else {
                this.log.debug("New endpoint, create it in the database");
                this.endpointDAO.create(session, edp);
            }

        } catch (DAOException e) {
            throw new RegistryException(e);
        } finally {
            if (session != null) {
                try {
                    session.disconnect();
                } catch (HibernateException e) {
                }
            }
        }
        return result;
    }

    public Collection<String> getKeys() {
        return null;
    }

    @Override
    protected long doSize() throws RegistryException {
        Session session = hibernateHelper.getSession();
        long result = 0L;
        try {
            result = this.endpointDAO.count(session,
                    org.ow2.petals.registry.core.repository.bo.Endpoint.class);
        } catch (DAOException e) {
            throw new RegistryException(e.getMessage());
        } finally {
            if (session != null) {
                try {
                    session.disconnect();
                } catch (HibernateException e) {
                }
            }
        }
        return result;
    }

    public long size(final String path) {
        return -1;
    }

    @Override
    protected List<Resource> doGetAll(final String path) throws RegistryException {
        Session session = hibernateHelper.getSession();
        List<Resource> result = null;
        List<Endpoint> endpoints = null;

        try {
            endpoints = this.endpointDAO.getAllFromPath(session, path);
        } catch (DAOException e) {
            throw new RegistryException(e.getMessage());
        } finally {
            if (session != null) {
                try {
                    session.disconnect();
                } catch (HibernateException e) {
                }
            }
        }

        if (endpoints != null) {
            result = new ArrayList<Resource>(endpoints.size());
            for (org.ow2.petals.registry.core.repository.bo.Endpoint e : endpoints) {
                result.add(this.adapter.toTO(e));
            }
        }
        return result;
    }

    @Override
    protected List<Resource> doQuery(final Query query) throws RegistryException {
        if (this.log.isDebugEnabled()) {
            this.log.debug(query);
        }

        Session session = hibernateHelper.getSession();
        List<Resource> result = null;
        List<Endpoint> endpoints = null;
        try {
            endpoints = this.endpointDAO.query(hibernateHelper, query);
        } catch (DAOException e) {
            throw new RegistryException(e.getMessage());
        } finally {
            if (session != null) {
                try {
                    session.disconnect();
                } catch (HibernateException e) {
                }
            }
        }

        if (endpoints != null) {
            result = new ArrayList<Resource>(endpoints.size());
            for (org.ow2.petals.registry.core.repository.bo.Endpoint e : endpoints) {
                result.add(this.adapter.toTO(e));
            }
        }
        return result;
    }

    @Override
    protected int doClean() throws RegistryException {
        int result = 0;
        Session session = hibernateHelper.getSession();
        try {
            result = this.endpointDAO.deleteAll(session);
        } catch (DAOException e) {
            throw new RegistryException(e.getMessage());
        } finally {
            if (session != null) {
                try {
                    session.disconnect();
                } catch (HibernateException e) {
                }
            }
        }
        return result;
    }

    @Override
    protected int doKeepOnly(final String registryName) throws RegistryException {
        int result = 0;
        Session session = hibernateHelper.getSession();

        String regName = registryName;
        if (registryName.endsWith(org.ow2.petals.registry.api.Constants.SEPARATOR)) {
            regName = registryName.substring(0, registryName.lastIndexOf(org.ow2.petals.registry.api.Constants.SEPARATOR));
        }

        try {
            result = this.endpointDAO.keepOnly(session, regName);
        } catch (DAOException e) {
            throw new RegistryException(e);
        } finally {
            if (session != null) {
                try {
                    session.disconnect();
                } catch (HibernateException e) {
                }
            }
        }
        return result;
    }
    
    @Override
    protected int doDeleteDataFromRegistry(final String registryName) throws RegistryException {
        int result = 0;
        Session session = hibernateHelper.getSession();
        try {
            result = this.endpointDAO.deleteFrom(session, registryName);
        } catch (DAOException e) {
            throw new RegistryException(e.getMessage());
        } finally {
            if (session != null) {
                try {
                    session.disconnect();
                } catch (HibernateException e) {
                }
            }
        }
        return result;
    }
    
    public void setHibernateHelper(HibernateHelper hibernateHelper) {
        this.hibernateHelper = hibernateHelper;
    }

    
    public HibernateHelper getHibernateHelper() {
        return hibernateHelper;
    }

    /**
     * @return the endpointDAO
     */
    public EndpointDAO getEndpointDAO() {
        return this.endpointDAO;
    }
}
