/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.registry.core.repository.jdbc;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.File;
import java.util.List;

import javax.xml.namespace.QName;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.ow2.petals.registry.api.Endpoint;
import org.ow2.petals.registry.api.Query;
import org.ow2.petals.registry.api.Resource;
import org.ow2.petals.registry.api.context.Context;
import org.ow2.petals.registry.api.context.ContextFactory;
import org.ow2.petals.registry.api.exception.RegistryException;

/**
 * @author Christophe HAMERLING - EBM WebSourcing
 */
public class JDBCRepositoryTest {

    static {
        BasicConfigurator.configure();
        Logger.getRootLogger().setLevel(Level.INFO);
        Logger logger = Logger.getLogger("org.hibernate");
        logger.setLevel(Level.INFO);
    }

    private static Log logger = LogFactory.getLog(JDBCRepositoryTest.class);

    private JDBCRepository repository;

    private static final QName DEFAULT_INTERFACE = QName.valueOf("defaultinterface");

    private static final QName DEFAULT_SERVICE = QName.valueOf("defaultservice");

    private static final String DEFAULT_NAME = "defaultname";

    QName ep1 = QName.valueOf("ep-" + System.currentTimeMillis());

    QName ep2 = QName.valueOf("ep-" + System.currentTimeMillis());

    QName srv1 = QName.valueOf("srv-" + System.currentTimeMillis());

    QName srv2 = QName.valueOf("srv2-" + System.currentTimeMillis());

    QName itf1 = QName.valueOf("itf1-" + System.currentTimeMillis());

    QName itf2 = QName.valueOf("itf2-" + System.currentTimeMillis());

    @Before
    public void setUp() throws Exception {
        logger.info("Start registry");
        File f = new File(".");
        File conf01 = new File(f, "src/test/resources/hsqlreg01.cfg");
        if (!conf01.exists()) {
            System.err.println("Can not find configuration resource");
            throw new Exception("Config file not found...");
        }

        Context context = ContextFactory.getInstance().createContext(conf01);
        this.repository = new JDBCRepository();

        this.repository.init(context);
        this.repository.start();
    }

    @After
    public void tearDown() throws Exception {
        logger.info("Stop registry");
        // clean all...
        try {
            this.repository.clean();
        } catch (Exception e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        this.repository.stop();
        this.repository = null;
    }

    @Test
    public void testSize() {
        try {
            assertEquals(0, this.repository.size());
        } catch (RegistryException e) {
            e.printStackTrace();
            fail(e.getMessage());
        }
    }

    /**
     * test a single put to see that this base operation works...
     * 
     * @throws Exception
     */
    @Test
    public void testPut() throws Exception {
        Resource resource = createDefaultResource();
        try {
            this.repository.put("/path", resource, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }
    }

    @Test
    public void testPutThenGet() {
        QName qname = QName.valueOf("myqname");
        Resource resource = createDefaultResource();
        resource.getEndpoint().setName(qname);
        try {
            this.repository.put("/path", resource, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        try {
            Resource result = this.repository.get("/path", false);
            assertEquals(qname.getLocalPart(), result.getEndpoint().getName());
        } catch (RegistryException e) {
            fail(e.getMessage());
        }
    }

    @Test
    public void testPutWithBadPath() throws Exception {

        QName qname = QName.valueOf("myqname");
        Resource resource = createDefaultResource();
        resource.getEndpoint().setName(qname);
        try {
            this.repository.put("/path/", resource, false);
            fail();
        } catch (RegistryException e) {
            logger.info(e.getMessage());
        }
    }

    @Test
    public void testPutThenGetAll() {
        for (int i = 0; i < 10; i++) {
            logger.info("PUT " + i);
            QName qname = QName.valueOf("myqname" + i);
            Resource resource = createDefaultResource();
            resource.getEndpoint().setName(qname);
            try {
                this.repository.put("/path/" + i, resource, false);
            } catch (RegistryException e) {
                logger.info(e.getMessage());
                fail();
            }
        }

        try {
            List<Resource> list = this.repository.getAll("/", false);
            assertEquals(10, list.size());

            list = this.repository.getAll("/path/", false);
            assertEquals(10, list.size());

            list = this.repository.getAll("/badpath/", false);
            assertEquals(0, list.size());

        } catch (RegistryException e) {
            fail(e.getMessage());
        }
    }

    @Test
    public void testPutThenDelete() {
        QName qname = QName.valueOf("myqname");
        String path = "/putthendelete";
        Resource resource = createDefaultResource();
        resource.getEndpoint().setName(qname);
        try {
            this.repository.put(path, resource, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        try {
            boolean result = this.repository.delete(path, false);
            assertTrue(result);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        try {
            Resource result = this.repository.get(path, false);
            assertNull(result);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }
    }

    @Test
    public void testUpdate() throws Exception {
        QName qname = QName.valueOf("myqname1");
        QName qname2 = QName.valueOf("myqname2");

        String path = "/update";
        Resource resource = createDefaultResource();
        resource.getEndpoint().setName(qname);
        try {
            this.repository.put(path, resource, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        Resource resource2 = createDefaultResource();
        resource2.getEndpoint().setName(qname2);

        try {
            this.repository.put(path, resource2, false);
        } catch (RegistryException e) {
            e.printStackTrace();
            fail(e.getMessage());
        }

        Resource get = null;
        try {
            get = this.repository.get(path, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        assertNotNull(get);
        assertEquals(qname2.getLocalPart(), get.getEndpoint().getName());

    }

    @Test
    public void testQueryEndpoint() {
        List<Resource> result = null;
        String path = "/query";
        Resource resource = createDefaultResource();
        resource.getEndpoint().setName(this.ep1);
        resource.getEndpoint().setService(this.srv1);
        resource.getEndpoint().setInterface(this.itf1);

        try {
            this.repository.put(path, resource, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        // search on name
        Query query = new Query();
        query.setEndpoint(this.ep1);
        try {
            result = this.repository.query(query, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        assertNotNull(result);
        assertEquals(1, result.size());
        assertEquals(result.get(0).getEndpoint().getName().toString(), this.ep1.toString());
    }

    @Test
    public void testQueryService() {
        List<Resource> result = null;
        String path = "/query";
        Resource resource = createDefaultResource();
        resource.getEndpoint().setName(this.ep1);
        resource.getEndpoint().setService(this.srv1);
        resource.getEndpoint().setInterface(this.itf1);

        try {
            this.repository.put(path, resource, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }
        // search on srv
        Query query = new Query();
        query.setService(this.srv1);
        try {
            result = this.repository.query(query, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        assertNotNull(result);
        assertEquals(1, result.size());
        assertEquals(result.get(0).getEndpoint().getName().toString(), this.ep1.toString());
    }

    /**
     * 
     */
    @Test
    public void testQueryOnSrvAndParams() {
        List<Resource> result = null;
        String path = "/queryparams1";

        Resource resource = createDefaultResource();
        resource.getEndpoint().setName(this.ep1);
        resource.getEndpoint().setService(this.srv1);
        resource.getEndpoint().setInterface(this.itf1);
        resource.getEndpoint().setContainer("1");

        try {
            this.repository.put(path, resource, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }
        // search on srv
        Query query = new Query();
        query.setService(this.srv1);
        query.setContainer("1");

        try {
            result = this.repository.query(query, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        assertNotNull(result);
        assertEquals(1, result.size());
        assertEquals(result.get(0).getEndpoint().getName().toString(), this.ep1.toString());

    }

    @Test
    public void testQueryOnSrvAndBadParam() {
        List<Resource> result = null;
        String path = "/queryparams1";

        Resource resource = createDefaultResource();
        resource.getEndpoint().setName(this.ep1);
        resource.getEndpoint().setService(this.srv1);
        resource.getEndpoint().setInterface(this.itf1);
        resource.getEndpoint().setContainer("1");

        try {
            this.repository.put(path, resource, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        Query query2 = new Query();
        query2.setService(this.srv1);
        query2.setContainer("2");

        try {
            result = this.repository.query(query2, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        assertNotNull(result);
        assertEquals(0, result.size());
    }

    @Test
    public void testQueryOnSrvsAndOneParam() {
        List<Resource> result = null;
        String path = "/queryparams1";

        Resource resource = createDefaultResource();
        Endpoint endpoint = resource.getEndpoint();
        endpoint.setName(this.ep1);
        endpoint.setService(this.srv1);
        endpoint.setInterface(this.itf1);
        endpoint.setContainer("1");

        try {
            this.repository.put(path, resource, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        resource = createDefaultResource();
        endpoint = resource.getEndpoint();
        endpoint.setName(this.ep1);
        endpoint.setService(this.srv1);
        endpoint.setInterface(this.itf1);
        endpoint.setContainer("1");
        endpoint.setSubdomain("A");
        resource.setEndpoint(endpoint);

        try {
            this.repository.put("/123", resource, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        Query query = new Query();
        query.setService(this.srv1);
        query.setContainer("1");

        // query 1 should return 2
        try {
            result = this.repository.query(query, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        assertNotNull(result);
        assertEquals(2, result.size());

    }

    @Test
    public void testQueryOnSrvsAndBadParam() {
        List<Resource> result = null;
        String path = "/queryparams1";

        Resource resource = createDefaultResource();
        Endpoint endpoint = resource.getEndpoint();
        endpoint.setName(this.ep1);
        endpoint.setService(this.srv2);
        endpoint.setInterface(this.itf1);
        endpoint.setContainer("1");

        try {
            this.repository.put(path, resource, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        resource = createDefaultResource();
        endpoint = resource.getEndpoint();
        endpoint.setName(this.ep1);
        endpoint.setService(this.srv1);
        endpoint.setInterface(this.itf1);
        endpoint.setContainer("1");
        endpoint.setSubdomain("A");

        try {
            this.repository.put("/123", resource, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        // query 2 should return 0 again
        Query query2 = new Query();
        query2.setService(this.srv1);
        query2.setContainer("2");

        try {
            result = this.repository.query(query2, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        assertNotNull(result);
        assertEquals(0, result.size());

    }

    @Test
    public void testQueryOnTwoParams() {
        List<Resource> result = null;
        String path = "/queryparams1";

        Resource resource = createDefaultResource();
        Endpoint endpoint = resource.getEndpoint();
        endpoint.setName(this.ep1);
        endpoint.setService(this.srv1);
        endpoint.setInterface(this.itf1);
        endpoint.setContainer("1");
        endpoint.setSubdomain("A");

        try {
            this.repository.put(path, resource, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        resource = createDefaultResource();
        endpoint = resource.getEndpoint();
        endpoint.setName(this.ep1);
        endpoint.setService(this.srv1);
        endpoint.setInterface(this.itf1);
        endpoint.setContainer("1");
        endpoint.setSubdomain("A");

        try {
            this.repository.put("/123", resource, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        // query on container and domain
        Query containerDomainQuery = new Query();
        containerDomainQuery.setService(this.srv1);
        containerDomainQuery.setContainer("1");
        containerDomainQuery.setSubDomain("A");

        try {
            result = this.repository.query(containerDomainQuery, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        assertNotNull(result);
        assertEquals(2, result.size());

    }

    @Test
    public void testQueryOnOneParams() {
        List<Resource> result = null;
        String path = "/queryparams1";

        Resource resource = createDefaultResource();
        Endpoint endpoint = resource.getEndpoint();
        endpoint.setName(this.ep1);
        endpoint.setService(this.srv1);
        endpoint.setInterface(this.itf1);
        endpoint.setContainer("1");
        endpoint.setSubdomain("A");

        try {
            this.repository.put(path, resource, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        resource = createDefaultResource();
        endpoint = resource.getEndpoint();
        endpoint.setName(this.ep1);
        endpoint.setService(this.srv1);
        endpoint.setInterface(this.itf1);
        endpoint.setContainer("1");
        endpoint.setSubdomain("A");
        resource.setEndpoint(endpoint);

        try {
            this.repository.put("/123", resource, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        // query on domain
        Query domainQuery = new Query();
        domainQuery.setService(this.srv1);
        domainQuery.setSubDomain("A");

        try {
            result = this.repository.query(domainQuery, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        assertNotNull(result);
        assertEquals(2, result.size());
    }

    @Test
    public void testQueryAndIgnoreParams() {
        List<Resource> result = null;
        String path = "/queryparams1";

        Resource resource = createDefaultResource();
        Endpoint endpoint = resource.getEndpoint();
        endpoint.setName(this.ep1);
        endpoint.setService(this.srv1);
        endpoint.setInterface(this.itf1);
        endpoint.setContainer("1");
        resource.setEndpoint(endpoint);

        try {
            this.repository.put(path, resource, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        resource = createDefaultResource();
        endpoint = resource.getEndpoint();
        endpoint.setName(this.ep1);
        endpoint.setService(this.srv1);
        endpoint.setInterface(this.itf1);
        endpoint.setContainer("1");
        endpoint.setSubdomain("A");
        resource.setEndpoint(endpoint);

        try {
            this.repository.put("/123", resource, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        // query all whitout properties
        Query q = new Query();
        q.setService(this.srv1);

        try {
            result = this.repository.query(q, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        assertNotNull(result);
        assertEquals(2, result.size());

    }

    public void deprecatedTestQueryOnlyWithParams() {
        List<Resource> result = null;
        String path = "/queryparams1";

        Resource resource = createDefaultResource();
        Endpoint endpoint = resource.getEndpoint();
        endpoint.setName(this.ep1);
        endpoint.setService(this.srv1);
        endpoint.setInterface(this.itf1);
        endpoint.setContainer("1");
        resource.setEndpoint(endpoint);

        try {
            this.repository.put(path, resource, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        resource = createDefaultResource();
        endpoint = resource.getEndpoint();
        endpoint.setName(this.ep1);
        endpoint.setService(this.srv1);
        endpoint.setInterface(this.itf1);
        endpoint.setContainer("1");
        endpoint.setSubdomain("A");
        resource.setEndpoint(endpoint);

        try {
            this.repository.put("/123", resource, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        Query q = new Query();
        q.setContainer("1");

        try {
            result = this.repository.query(q, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        assertNotNull(result);
        assertEquals(2, result.size());

    }

    @Test
    public void testQueryInterfaceAndType() {
        List<Resource> result = null;
        String path = "/query";
        Resource resource = createDefaultResource();
        Endpoint endpoint = resource.getEndpoint();
        endpoint.setName(this.ep1);
        endpoint.setService(this.srv1);
        endpoint.setInterface(this.itf1);
        endpoint.setType("mytype");

        resource.setEndpoint(endpoint);
        try {
            this.repository.put(path, resource, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        // search on one itf
        Query query = new Query();
        query.setInterface(this.itf1);
        query.setType("mytype");

        try {
            result = this.repository.query(query, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        assertNotNull(result);
        assertEquals(1, result.size());
        assertEquals(result.get(0).getEndpoint().getName().toString(), this.ep1.toString());

        // search on bad type
        query = new Query();
        query.setInterface(this.itf1);
        query.setType("badtype");

        try {
            result = this.repository.query(query, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        assertNotNull(result);
        assertEquals(0, result.size());
    }

    @Test
    public void testQueryInterface() {
        List<Resource> result = null;
        String path = "/query";
        Resource resource = createDefaultResource();
        Endpoint endpoint = resource.getEndpoint();
        endpoint.setName(this.ep1);
        endpoint.setService(this.srv1);
        endpoint.setInterface(this.itf1);

        resource.setEndpoint(endpoint);
        try {
            this.repository.put(path, resource, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        // search on one itf
        Query query = new Query();
        query.setInterface(this.itf1);

        try {
            result = this.repository.query(query, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        assertNotNull(result);
        assertEquals(1, result.size());
        assertEquals(result.get(0).getEndpoint().getName().toString(), this.ep1.toString());

        // search on bad interface
        query = new Query();
        query.setInterface(QName.valueOf("BADINTERFACE"));

        try {
            result = this.repository.query(query, false);
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        assertNotNull(result);
        assertEquals(0, result.size());
    }

    @Test
    public void testKeepOnly() {
        for (int i = 0; i < 5; i++) {
            Resource resource = createDefaultResource();
            resource.setRootRegistry("registry1");
            Endpoint endpoint = resource.getEndpoint();
            endpoint.setName(QName.valueOf("myqname"));
            resource.setEndpoint(endpoint);
            try {
                this.repository.put("/path/1/" + i, resource, false);
            } catch (RegistryException e) {
                fail(e.getMessage());
            }
        }

        long size = 0;
        try {
            size = this.repository.size();
        } catch (RegistryException e) {
            fail(e.getMessage());
        }
        assertEquals(5, size);

        for (int i = 0; i < 3; i++) {
            Resource resource = createDefaultResource();
            resource.setRootRegistry("registry2");
            Endpoint endpoint = resource.getEndpoint();
            endpoint.setName(QName.valueOf("myqname"));
            resource.setEndpoint(endpoint);
            try {
                this.repository.put("/path/2/" + i, resource, false);
            } catch (RegistryException e) {
                fail(e.getMessage());
            }
        }

        try {
            size = this.repository.size();
        } catch (RegistryException e) {
            fail(e.getMessage());
        }
        assertEquals(8, size);

        for (int i = 0; i < 2; i++) {
            Resource resource = createDefaultResource();
            resource.setRootRegistry("registry3");
            Endpoint endpoint = resource.getEndpoint();
            endpoint.setName(QName.valueOf("myqname"));
            resource.setEndpoint(endpoint);
            try {
                this.repository.put("/path/3/" + i, resource, false);
            } catch (RegistryException e) {
                fail(e.getMessage());
            }
        }

        try {
            size = this.repository.size();
        } catch (RegistryException e) {
            fail(e.getMessage());
        }
        assertEquals(10, size);

        // let's delete things...
        int deleted = 0;
        try {
            deleted = this.repository.keepOnly("registry1");
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        assertEquals(deleted, 5);

        try {
            size = this.repository.size();
        } catch (RegistryException e) {
            fail(e.getMessage());
        }
        assertEquals(5, size);

        // delete all
        try {
            deleted = this.repository.keepOnly("registry123");
        } catch (RegistryException e) {
            fail(e.getMessage());
        }

        assertEquals(deleted, 5);

        try {
            size = this.repository.size();
        } catch (RegistryException e) {
            fail(e.getMessage());
        }
        assertEquals(0, size);
    }

    @Test
    public void testDeletePath() throws Exception {
        for (int i = 0; i < 5; i++) {
            Resource resource = createDefaultResource();
            resource.setRootRegistry("registry1");
            Endpoint endpoint = resource.getEndpoint();
            endpoint.setName(QName.valueOf("myqname"));
            resource.setEndpoint(endpoint);
            try {
                this.repository.put("/path/1/" + i, resource, false);
            } catch (RegistryException e) {
                fail(e.getMessage());
            }
        }

        long size = 0;
        try {
            size = this.repository.size();
        } catch (RegistryException e) {
            fail(e.getMessage());
        }
        assertEquals(5, size);

        this.repository.delete("/path/", true);

        try {
            size = this.repository.size();
        } catch (RegistryException e) {
            fail(e.getMessage());
        }
        assertEquals(0, size);
    }

    private static Endpoint createDefaultEndpoint() {
        Endpoint endpoint = new Endpoint();
        endpoint.setInterface(DEFAULT_INTERFACE);
        endpoint.setName(DEFAULT_NAME);
        endpoint.setService(DEFAULT_SERVICE);
        endpoint.setContainer("container");
        endpoint.setComponent("component");
        endpoint.setSubdomain("subdomain");
        endpoint.setType("type");
        return endpoint;
    }

    private static Resource createDefaultResource() {
        Resource resource = new Resource();
        resource.setRootRegistry("rootregistry");
        resource.setEndpoint(createDefaultEndpoint());
        return resource;
    }

}
